BASEBALL_REFERENCE_SEASON_URL_PREFIX = 'https://www.baseball-reference.com/leagues'

BASEBALL_REFERENCE_LEAUGE_TABLE_CONFIGS = [
    {
        'table': 'Team Wins',
        'table_identifier': 'teams_team_wins3000',
        'url_prefix': BASEBALL_REFERENCE_SEASON_URL_PREFIX,
        'league_required': True,
        'year_required': False,
        'url_postfix': '',
        'shtml_postfix_required': False,
        'remove_rows_on': []     
    }
]

BASEBALL_REFERENCE_LEAUGE_YEAR_CONFIGS = [
    {
        'table': 'East Division',
        'table_identifier': 'standings_E',
        'url_prefix': BASEBALL_REFERENCE_SEASON_URL_PREFIX,
        'league_required': True,
        'year_required': True,
        'url_postfix': '',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    },
    {
        'table': 'Central Division',
        'table_identifier': 'standings_C',
        'url_prefix': BASEBALL_REFERENCE_SEASON_URL_PREFIX,
        'league_required': True,
        'year_required': True,
        'url_postfix': '',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    },
    {
        'table': 'West Division',
        'table_identifier': 'standings_W',
        'url_prefix': BASEBALL_REFERENCE_SEASON_URL_PREFIX,
        'league_required': True,
        'year_required': True,
        'url_postfix': '',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    },
    {
        'table': 'Postseason',
        'table_identifier': 'postseason',
        'url_prefix': BASEBALL_REFERENCE_SEASON_URL_PREFIX,
        'league_required': True,
        'year_required': True,
        'url_postfix': '',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    },
    {
        'table': 'Team Standard Batting',
        'table_identifier': 'teams_standard_batting',
        'url_prefix': BASEBALL_REFERENCE_SEASON_URL_PREFIX,
        'league_required': True,
        'year_required': True,
        'url_postfix': '',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    },
    {
        'table': 'Team Standard Pitching',
        'table_identifier': 'teams_standard_pitching',
        'url_prefix': BASEBALL_REFERENCE_SEASON_URL_PREFIX,
        'league_required': True,
        'year_required': True,
        'url_postfix': '',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    },
    {
        'table': 'American League Wins Above Average By Position',
        'table_identifier': 'team_output',
        'url_prefix': BASEBALL_REFERENCE_SEASON_URL_PREFIX,
        'league_required': True,
        'year_required': True,
        'url_postfix': '',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    },
    {
        'table': 'National League Wins Above Average By Position',
        'table_identifier': 'team_output',
        'url_prefix': BASEBALL_REFERENCE_SEASON_URL_PREFIX,
        'league_required': True,
        'year_required': True,
        'url_postfix': '',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    },
    {
        'table': 'Team Fielding',
        'table_identifier': 'teams_standard_fielding',
        'url_prefix': BASEBALL_REFERENCE_SEASON_URL_PREFIX,
        'league_required': True,
        'year_required': True,
        'url_postfix': '',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    },
    {
        'table': 'Major League Baseball Wins Above Avg By Position',
        'table_identifier': 'team_output',
        'url_prefix': BASEBALL_REFERENCE_SEASON_URL_PREFIX,
        'league_required': True,
        'year_required': True,
        'url_postfix': '',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    }
]

BASEBALL_REFERENCE_LEAUGE_YEAR_STANDINGS_CONFIGS = [
    {
        'table': 'American League East Detailed Standings',
        'table_identifier': 'expanded_standings_E',
        'url_prefix': BASEBALL_REFERENCE_SEASON_URL_PREFIX,
        'league_required': True,
        'year_required': True,
        'url_postfix': '-standings',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    },
    {
        'table': 'American League Central Detailed Standings',
        'table_identifier': 'expanded_standings_C',
        'url_prefix': BASEBALL_REFERENCE_SEASON_URL_PREFIX,
        'league_required': True,
        'year_required': True,
        'url_postfix': '-standings',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    },
    {
        'table': 'American League West Detailed Standings',
        'table_identifier': 'expanded_standings_W',
        'url_prefix': BASEBALL_REFERENCE_SEASON_URL_PREFIX,
        'league_required': True,
        'year_required': True,
        'url_postfix': '-standings',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    },
    {
        'table': 'American League Wild Card Detailed Standings',
        'table_identifier': 'expanded_standings_wild_card',
        'url_prefix': BASEBALL_REFERENCE_SEASON_URL_PREFIX,
        'league_required': True,
        'year_required': True,
        'url_postfix': '-standings',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    },
    {
        'table': 'Head-To-Head Records',
        'table_identifier': 'head_to_head_wins',
        'url_prefix': BASEBALL_REFERENCE_SEASON_URL_PREFIX,
        'league_required': True,
        'year_required': True,
        'url_postfix': '-standings',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    },
    {
        'table': 'Head to Head Run Scoring',
        'table_identifier': 'head_to_head_runs',
        'url_prefix': BASEBALL_REFERENCE_SEASON_URL_PREFIX,
        'league_required': True,
        'year_required': True,
        'url_postfix': '-standings',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    },
    {
        'table': 'National League East Detailed Standings',
        'table_identifier': 'expanded_standings_E',
        'url_prefix': BASEBALL_REFERENCE_SEASON_URL_PREFIX,
        'league_required': True,
        'year_required': True,
        'url_postfix': '-standings',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    },
    {
        'table': 'National League Central Detailed Standings',
        'table_identifier': 'expanded_standings_C',
        'url_prefix': BASEBALL_REFERENCE_SEASON_URL_PREFIX,
        'league_required': True,
        'year_required': True,
        'url_postfix': '-standings',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    },
    {
        'table': 'National League West Detailed Standings',
        'table_identifier': 'expanded_standings_W',
        'url_prefix': BASEBALL_REFERENCE_SEASON_URL_PREFIX,
        'league_required': True,
        'year_required': True,
        'url_postfix': '-standings',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    },
    {
        'table': 'National League Wild Card Detailed Standings',
        'table_identifier': 'expanded_standings_wild_card',
        'url_prefix': BASEBALL_REFERENCE_SEASON_URL_PREFIX,
        'league_required': True,
        'year_required': True,
        'url_postfix': '-standings',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    }
]

BASEBALL_REFERENCE_LEAUGE_YEAR_STANDARD_FIELDING_CONFIGS = [
    {
        'table': 'Player Standard Fielding',
        'table_identifier': 'players_players_standard_fielding_fielding',
        'url_prefix': BASEBALL_REFERENCE_SEASON_URL_PREFIX,
        'league_required': True,
        'year_required': True,
        'url_postfix': '-standard-fielding',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    }
]

BASEBALL_REFERENCE_LEAUGE_YEAR_STANDARD_BATTING_CONFIGS = [
    {
        'table': 'Player Standard Batting',
        'table_identifier': 'players_standard_batting',
        'url_prefix': BASEBALL_REFERENCE_SEASON_URL_PREFIX,
        'league_required': True,
        'year_required': True,
        'url_postfix': '-standard-batting',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    }
]

BASEBALL_REFERENCE_LEAUGE_YEAR_STANDARD_PITCHING_CONFIGS = [
    {
        'table': 'Player Standard Pitching',
        'table_identifier': 'players_standard_pitching',
        'url_prefix': BASEBALL_REFERENCE_SEASON_URL_PREFIX,
        'league_required': True,
        'year_required': True,
        'url_postfix': '-standard-pitching',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    }
]

BASEBALL_REFERENCE_SEASON_TABLE_CONFIGS = [
    *BASEBALL_REFERENCE_LEAUGE_TABLE_CONFIGS,
    *BASEBALL_REFERENCE_LEAUGE_YEAR_CONFIGS,
    *BASEBALL_REFERENCE_LEAUGE_YEAR_STANDINGS_CONFIGS,
    *BASEBALL_REFERENCE_LEAUGE_YEAR_STANDARD_FIELDING_CONFIGS,
    *BASEBALL_REFERENCE_LEAUGE_YEAR_STANDARD_BATTING_CONFIGS,
    *BASEBALL_REFERENCE_LEAUGE_YEAR_STANDARD_PITCHING_CONFIGS
]


from const import data_source_const
from baseball_reference import baseball_referenence_request_handler
from request_validation import request_validator
 
def get_data(baseballdc_request):

    # Validate that the incoming request payload has a valid data source.
    request_validator.validate_incoming_payload(baseballdc_request)

    # Route the request to the correct data source handler, and generate the requested dataframe
    df = route_get_data_request(baseballdc_request)

    ## for now printing, should return df for lib
    print(df)

def route_get_data_request(baseballdc_request):
    requested_data_source_trimmed = baseballdc_request['data_source'].strip().upper()

    if(requested_data_source_trimmed == data_source_const.BASEBALL_REFERENCE):
        df = baseball_referenence_request_handler.get_baseball_reference_data(baseballdc_request)

    return df


# # EXAMPLE REQUEST
# baseballdc_request = {
# 	'data_source': 'BASEBALL_REFERENCE',
# 	'query_params': {
#         'scope': 'TEAM',
#         'table': 'Year-by-Year Team Pitching Ranks',
#         'team': 'DET',
#         'year': 2002
# 	}
# }

# baseballdc_request_season = {
# 	'data_source': 'BASEBALL_REFERENCE',
# 	'query_params': {
#         'scope': 'SEASON',
#         'table': 'Player Standard Pitching',
#         # 'team': 'DET',
#         'year': 2003,
#         'league': 'NL'
# 	}
# }

# baseballdc_request_player = {
# 	'data_source': 'BASEBALL_REFERENCE',
# 	'query_params': {
#         'scope': 'INDIVIDUAL_PLAYER',
#         'table': 'Standard Batting',
#         'first_name': 'Miguel',
#         'last_name': "Cabrera"
# 	}
# }

# baseballdc_request_player = {
# 	'data_source': 'BASEBALL_REFERENCE',
# 	'query_params': {
#         'scope': 'INDIVIDUAL_PLAYER',
#         'table': 'Standard Batting',
#         'first_name': 'Justin',
#         'last_name': 'Verlander'
# 	}
# }


# get_data(baseballdc_request_player)


def get_table_config(query_params, table_configs):

    if 'table' not in query_params:
        raise ValueError(generate_no_table_in_params_error_message(query_params))

    table_config = None
    for config in table_configs:
        if(compare_table_names(query_params['table'], config['table'])):
            table_config = config
    
    if(table_config is None):
        raise ValueError(generate_table_config_error_message(query_params))

    return table_config

def compare_table_names(user_input_value, config_value):
    user_input_value_trimmed = ''.join(e for e in user_input_value if e.isalpha()).upper()
    config_value_trimmed = ''.join(e for e in config_value if e.isalpha()).upper()

    if(user_input_value_trimmed == config_value_trimmed):
        return True
    else: 
        return False

def generate_no_table_in_params_error_message(query_params): 

    error_message_text = 'Value error in the incoming request payload:'
    query_params_error_text = f'The query parameters included in the incoming request must contain a "table" key-value pair.'

    return f'{error_message_text}\n\n{query_params_error_text}\n';

def generate_table_config_error_message(query_params): 

    requested_table = query_params['table']
    requested_scope = query_params['scope']

    error_message_text = 'Value error in the incoming request payload:'
    data_source_error_text = f'The table requested ("{requested_table}") could not be found in the {requested_scope} scope.'
    report_text = f'If you believe this to be an error, please report this as an issue @ https://github.com/joesmi9/baseballdc.'

    return f'{error_message_text}\n\n{data_source_error_text}\n{report_text}\n';


BASEBALL_REFERENCE_TEAMS_URL_PREFIX = 'https://www.baseball-reference.com/teams'

'''
All Franchise Tables
'''
BASEBALL_REFERENCE_ALL_FRANCHISE_TABLE_CONFIGS = [
    {
        'table': 'Active Franchises',
        'table_identifier': 'teams_active',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': False,
        'year_required': False,
        'url_postfix': '',
        'shtml_postfix_required': False,
        'remove_rows_on': ['Rk:Rk']     
    },
    {
        'table': 'Inactive Franchises',
        'table_identifier': 'teams_defunct',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': False,
        'year_required': False,
        'url_postfix': '',
        'shtml_postfix_required': False,   
        'remove_rows_on': ['Rk:Rk']     
    },
    {
        'table': 'National Association Franchises',
        'table_identifier': 'teams_na',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': False,
        'year_required': False,
        'url_postfix': '',
        'shtml_postfix_required': False,
        'remove_rows_on': []     
    }
]

'''
Single Franchise History Tables
'''
BASEBALL_REFERENCE_FRANCHISE_TABLE_CONFIGS = [
    {
        'table': 'Franchise History',
        'table_identifier': 'franchise_years',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': False,
        'url_postfix': '',
        'shtml_postfix_required': False,   
        'remove_rows_on': ['Year:Year'] 
    }
]

'''
Single Franchise w/ Year Tables
'''
BASEBALL_REFERENCE_TEAM_YEAR_TABLE_CONFIGS = [
    {
        'table': 'Team Batting',
        'table_identifier': 'team_batting',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Rk:Rk']     
    },
    {
        'table': 'Team Pitching',
        'table_identifier': 'team_pitching',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Rk:Rk']     
    },
    {
        'table': 'Full-Season Roster & Games by Position',
        'table_identifier': 'appearances',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Name:Name']            
    },
    {
        'table': 'Coaching Staff',
        'table_identifier': 'coaches',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Name:Name']            
    },
    {
        'table': 'Team Fielding Totals',
        'table_identifier': 'standard_fielding',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Name:Name']            
    },    
    {
        'table': 'Team Player Value--Batters',
        'table_identifier': 'players_value_batting',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Name:Name']            
    },
    {
        'table': 'Team Player Value--Pitchers',
        'table_identifier': 'players_value_pitching',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Name:Name']            
    }
]

'''
Single Franchise w/ Year Batting Tables
'''
BASEBALL_REFERENCE_TEAM_YEAR_BATTING_TABLE_CONFIGS = [
    {
        'table': 'Team Advanced Batting',
        'table_identifier': 'players_advanced_batting',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-batting',
        'shtml_postfix_required': True,   
        'remove_rows_on': []            
    },
    {
        'table': 'Team Sabermetric Batting',
        'table_identifier': 'players_sabermetric_batting',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-batting',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Name:Name']            
    },
    {
        'table': 'Team Batting Ratios*',
        'table_identifier': 'players_ratio_batting',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-batting',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Name:Name']            
    },
    {
        'table': 'Team Win Probability*',
        'table_identifier': 'players_win_probability_batting',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-batting',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Name:Name']            
    }, 
    {
        'table': 'Team Baserunning/Misc*',
        'table_identifier': 'players_baserunning_batting',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-batting',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Name:Name']            
    }, 
    {
        'table': 'Team PH/HR/Situ Hitting*',
        'table_identifier': 'players_situational_batting',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-batting',
        'shtml_postfix_required': True,   
        'remove_rows_on': []            
    }, 
    {
        'table': 'Team Pitches Batting*',
        'table_identifier': 'players_pitches_batting',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-batting',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Name:Name']            
    },
    {
        'table': 'Team Neutralized Batting',
        'table_identifier': 'players_neutral_batting',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-batting',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Name:Name']            
    },
    {
        'table': 'Team Cumulative Batting',
        'table_identifier': 'players_cumulative_batting',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-batting',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Name:Name']            
    }    
]

'''
Single Franchise w/ Year Pitching Tables
'''
BASEBALL_REFERENCE_TEAM_YEAR_PITCHING_TABLE_CONFIGS = [
    {
        'table': 'Team Advanced Pitching',
        'table_identifier': 'players_pitching_advanced',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-pitching',
        'shtml_postfix_required': True,   
        'remove_rows_on': []            
    },
    {
        'table': 'Team Pitching Ratios*',
        'table_identifier': 'players_ratio_pitching',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-pitching',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Name:Name']            
    },
    {
        'table': 'Team Batting Against*',
        'table_identifier': 'players_batting_pitching',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-pitching',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Name:Name']            
    },
    {
        'table': 'Team Win Probability*',
        'table_identifier': 'players_win_probability_pitching',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-pitching',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Name:Name']            
    },
    {
        'table': 'Team Starting Pitching*',
        'table_identifier': 'players_starter_pitching',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-pitching',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Name:Name']            
    },
    {
        'table': 'Team Relief Pitching*',
        'table_identifier': 'players_reliever_pitching',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-pitching',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Name:Name']            
    },
    {
        'table': 'Team Baserunning/Situ*',
        'table_identifier': 'players_basesituation_pitching',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-pitching',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Name:Name']            
    },
    {
        'table': 'Team Pitching Pitches*',
        'table_identifier': 'players_pitches_pitching',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-pitching',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Name:Name']            
    },
    {
        'table': 'Team Neutralized Pitching',
        'table_identifier': 'players_neutral_pitching',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-pitching',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Name:Name']            
    },    
    {
        'table': 'Team Cumulative Pitching',
        'table_identifier': 'players_cumulative_pitching',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-pitching',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Name:Name']            
    },   
]

'''
Single Franchise w/ Year Fielding Tables
'''
BASEBALL_REFERENCE_TEAM_YEAR_FIELDING_TABLE_CONFIGS = [
    {
        'table': 'Player Standard Fielding--C',
        'table_identifier': 'players_standard_fielding_c',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-fielding',
        'shtml_postfix_required': True,   
        'remove_rows_on': []            
    },
    {
        'table': 'Player Standard Fielding--1B',
        'table_identifier': 'players_standard_fielding_1b',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-fielding',
        'shtml_postfix_required': True,   
        'remove_rows_on': []            
    },
    {
        'table': 'Player Standard Fielding--2B',
        'table_identifier': 'players_standard_fielding_2b',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-fielding',
        'shtml_postfix_required': True,   
        'remove_rows_on': []            
    },    
    {
        'table': 'Player Standard Fielding--3B',
        'table_identifier': 'players_standard_fielding_3b',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-fielding',
        'shtml_postfix_required': True,   
        'remove_rows_on': []            
    },
    {
        'table': 'Player Standard Fielding--SS',
        'table_identifier': 'players_standard_fielding_ss',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-fielding',
        'shtml_postfix_required': True,   
        'remove_rows_on': []            
    },
    {
        'table': 'Player Standard Fielding--LF',
        'table_identifier': 'players_standard_fielding_lf',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-fielding',
        'shtml_postfix_required': True,   
        'remove_rows_on': []            
    },
    {
        'table': 'Player Standard Fielding--CF',
        'table_identifier': 'players_standard_fielding_cf',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-fielding',
        'shtml_postfix_required': True,   
        'remove_rows_on': []            
    },
    {
        'table': 'Player Standard Fielding--RF',
        'table_identifier': 'players_standard_fielding_rf',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-fielding',
        'shtml_postfix_required': True,   
        'remove_rows_on': []            
    },
    {
        'table': 'Player Standard Fielding--OF',
        'table_identifier': 'players_standard_fielding_of',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-fielding',
        'shtml_postfix_required': True,   
        'remove_rows_on': []            
    },
    {
        'table': 'Player Standard Fielding--P',
        'table_identifier': 'players_standard_fielding_p',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-fielding',
        'shtml_postfix_required': True,   
        'remove_rows_on': []            
    },
    {
        'table': 'Player DH Games',
        'table_identifier': 'players_DH_games',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-fielding',
        'shtml_postfix_required': True,   
        'remove_rows_on': []            
    },
    {
        'table': 'Player Advanced Fielding -- C',
        'table_identifier': 'players_advanced_fielding_c',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-fielding',
        'shtml_postfix_required': True,   
        'remove_rows_on': []            
    },
    {
        'table': 'Player Advanced Fielding -- C Baserunning',
        'table_identifier': 'players_advanced_fielding_c_baserunning',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-fielding',
        'shtml_postfix_required': True,   
        'remove_rows_on': []            
    },
    {
        'table': 'Player Advanced Fielding -- 1B',
        'table_identifier': 'players_advanced_fielding_1b',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-fielding',
        'shtml_postfix_required': True,   
        'remove_rows_on': []            
    },
    {
        'table': 'Player Advanced Fielding -- 2B',
        'table_identifier': 'players_advanced_fielding_2b',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-fielding',
        'shtml_postfix_required': True,   
        'remove_rows_on': []            
    },
    {
        'table': 'Player Advanced Fielding -- 3B',
        'table_identifier': 'players_advanced_fielding_3b',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-fielding',
        'shtml_postfix_required': True,   
        'remove_rows_on': []            
    },
    {
        'table': 'Player Advanced Fielding -- SS',
        'table_identifier': 'players_advanced_fielding_ss',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-fielding',
        'shtml_postfix_required': True,   
        'remove_rows_on': []            
    },
    {
        'table': 'Player Advanced Fielding -- LF',
        'table_identifier': 'players_advanced_fielding_lf',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-fielding',
        'shtml_postfix_required': True,   
        'remove_rows_on': []            
    },
    {
        'table': 'Player Advanced Fielding -- CF',
        'table_identifier': 'players_advanced_fielding_cf',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-fielding',
        'shtml_postfix_required': True,   
        'remove_rows_on': []            
    },
    {
        'table': 'Player Advanced Fielding -- RF',
        'table_identifier': 'players_advanced_fielding_rf',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-fielding',
        'shtml_postfix_required': True,   
        'remove_rows_on': []            
    },
    {
        'table': 'Player Advanced Fielding -- P',
        'table_identifier': 'players_advanced_fielding_p',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': True,
        'url_postfix': '-fielding',
        'shtml_postfix_required': True,   
        'remove_rows_on': []            
    },
]

'''
Single Franchise w/ Year Fielding Tables
'''
BASEBALL_REFERENCE_FRANCHISE_BATTING_HISTORY_TABLE_CONFIGS = [
    {
        'table': 'Year-by-Year Team Batting',
        'table_identifier': 'yby_team_bat',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': False,
        'url_postfix': '/batteam',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Year:Year']     
    },
    {
        'table': 'Year-by-Year Team Batting per Game',
        'table_identifier': 'yby_team_bat_per_game',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': False,
        'url_postfix': '/batteam',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Year:Year']     
    },
    {
        'table': 'Year-by-Year Team Batting Ranks',
        'table_identifier': 'yby_team_bat_ranks',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': False,
        'url_postfix': '/batteam',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Year:Year']     
    },
]

'''
Single Franchise w/ Year Pitching Tables
'''
BASEBALL_REFERENCE_FRANCHISE_PITCHING_HISTORY_TABLE_CONFIGS = [
    {
        'table': 'Year-by-Year Team Pitching',
        'table_identifier': 'yby_team_pitch',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': False,
        'url_postfix': '/pitchteam',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Year:Year']            
    },
    {
        'table': 'Year-by-Year Team Pitching per Game',
        'table_identifier': 'yby_team_pitch_per_game',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': False,
        'url_postfix': '/pitchteam',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Year:Year']            
    },
    {
        'table': 'Year-by-Year Team Pitching Ranks',
        'table_identifier': 'yby_team_pitch_ranks',
        'url_prefix': BASEBALL_REFERENCE_TEAMS_URL_PREFIX,
        'team_required': True,
        'year_required': False,
        'url_postfix': '/pitchteam',
        'shtml_postfix_required': True,   
        'remove_rows_on': ['Year:Year']            
    },
]

BASEBALL_REFERENCE_TEAM_TABLE_CONFIGS = [
    *BASEBALL_REFERENCE_ALL_FRANCHISE_TABLE_CONFIGS, 
    *BASEBALL_REFERENCE_FRANCHISE_TABLE_CONFIGS, 
    *BASEBALL_REFERENCE_TEAM_YEAR_TABLE_CONFIGS,
    *BASEBALL_REFERENCE_TEAM_YEAR_BATTING_TABLE_CONFIGS,
    *BASEBALL_REFERENCE_TEAM_YEAR_PITCHING_TABLE_CONFIGS,
    *BASEBALL_REFERENCE_TEAM_YEAR_FIELDING_TABLE_CONFIGS,
    *BASEBALL_REFERENCE_FRANCHISE_BATTING_HISTORY_TABLE_CONFIGS,
    *BASEBALL_REFERENCE_FRANCHISE_PITCHING_HISTORY_TABLE_CONFIGS
]


def format_df(df, table_config):
    df = remove_duplicate_table_headers(df, table_config)

    return df

def remove_duplicate_table_headers(df, table_config):

    remove_rows_on = table_config['remove_rows_on']

    for entry in remove_rows_on:
        first_value = entry.partition(":")[0]
        second_value = entry.partition(":")[2]

        df = df.drop(df[df[first_value] == second_value].index)

    return df


import pandas as pd
from baseball_reference.const import baseball_reference_individual_player_tables_const
from baseball_reference.shared import table_config_service, table_format_service, table_request_service
from baseball_reference.individual_player import individual_player_url_service

def get_baseball_reference_individual_player_data(query_params):

    table_configs = baseball_reference_individual_player_tables_const.BASEBALL_REFERENCE_INDIVIDUAL_PLAYER_TABLE_CONFIGS
    table_config = table_config_service.get_table_config(query_params, table_configs)

    url = individual_player_url_service.construct_url(table_config, query_params)

    table = table_request_service.get_table(table_config, url, query_params)

    df = pd.read_html(str(table))[0]

    formatted_df = table_format_service.format_df(df, table_config)

    return formatted_df





import requests
from bs4 import BeautifulSoup, Comment

def get_table(table_config, url, query_params):
    response = requests.get(url, headers = {
        'accept-language':'en-US,en;q=0.8',
    })

    table = None

    table = get_commented_table(response, table_config)
    if(table == None):
        table = get_uncommented_table(response, table_config)
    
    if(table == None):
        raise ValueError(generate_table_error_message(query_params))

    return table
    

def get_uncommented_table(response, table_config):

    soup = BeautifulSoup(response.content, 'html5lib')

    table = soup.find('table', {'id': table_config['table_identifier']})

    return table


def get_commented_table(response, table_config):

    soup = BeautifulSoup(response.content, 'html5lib')

    id_plus_table_name = "".join(['id="', table_config['table_identifier'], '"'])

    commented_table = None;
    for comment in soup.findAll(text=lambda text:isinstance(text, Comment)):

        if(id_plus_table_name in comment):
            commented_table = comment

    if(commented_table != None):
        soup = BeautifulSoup(commented_table , 'html5lib')

        table = soup.find('table', {'id': table_config['table_identifier']})

        return table
    else:
        return None

def generate_table_error_message(query_params): 

    requested_table = query_params['table']
    requested_scope = query_params['scope']

    error_message_text = 'Value error in the incoming request payload:'
    data_source_error_text = f'The table requested ("{requested_table}") could not be found in the {requested_scope} scope.'
    report_text = f'If you believe this to be an error, please report this as an issue @ https://github.com/joesmi9/baseballdc.'

    return f'{error_message_text}\n\n{data_source_error_text}\n{report_text}\n';


import const.data_source_const as data_source_const

def validate_incoming_payload(baseballdc_request):

    requested_data_source_trimmed = baseballdc_request['data_source'].strip().upper()

    if(requested_data_source_trimmed not in data_source_const.DATA_SOURCE_LIST):
        raise ValueError(generate_data_source_error_message(baseballdc_request))

def generate_data_source_error_message(baseballdc_request): 

    requested_data_source = baseballdc_request['data_source']
    valid_data_sources = data_source_const.DATA_SOURCE_LIST

    error_message_text = 'Value error in the incoming request payload:'
    data_source_error_text = f'The data source requested ("{requested_data_source}") was not valid.'
    valid_data_source_text = f'The current list of valid datasources to choose from for basballdc is: {valid_data_sources}'

    return f'{error_message_text}\n\n{data_source_error_text}\n{valid_data_source_text}\n';



BASEBALL_REFERENCE_INDIVIDUAL_PLAYER_URL_PREFIX = 'https://www.baseball-reference.com/players'

BASEBALL_REFERENCE_PLAYER_TABLE_CONFIGS = [
    {
        'table': 'Standard Batting',
        'table_identifier': 'batting_standard',
        'url_prefix': BASEBALL_REFERENCE_INDIVIDUAL_PLAYER_URL_PREFIX,
        'first_name_required': True,
        'last_name_required': True,
        'url_postfix': '',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    },
    {
        'table': 'Player Value—Batting',
        'table_identifier': 'batting_value',
        'url_prefix': BASEBALL_REFERENCE_INDIVIDUAL_PLAYER_URL_PREFIX,
        'first_name_required': True,
        'last_name_required': True,
        'url_postfix': '',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    },
    {
        'table': 'Advanced Batting',
        'table_identifier': 'batting_advanced',
        'url_prefix': BASEBALL_REFERENCE_INDIVIDUAL_PLAYER_URL_PREFIX,
        'first_name_required': True,
        'last_name_required': True,
        'url_postfix': '',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    },
    {
        'table': 'Standard Fielding',
        'table_identifier': 'standard_fielding',
        'url_prefix': BASEBALL_REFERENCE_INDIVIDUAL_PLAYER_URL_PREFIX,
        'first_name_required': True,
        'last_name_required': True,
        'url_postfix': '',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    },
    {
        'table': 'Appearances',
        'table_identifier': 'appearances',
        'url_prefix': BASEBALL_REFERENCE_INDIVIDUAL_PLAYER_URL_PREFIX,
        'first_name_required': True,
        'last_name_required': True,
        'url_postfix': '',
        'shtml_postfix_required': True,
        'remove_rows_on': []     
    }
]

BASEBALL_REFERENCE_INDIVIDUAL_PLAYER_TABLE_CONFIGS = [
    *BASEBALL_REFERENCE_PLAYER_TABLE_CONFIGS
]


from baseball_reference.individual_player import individual_player_identifier_service

def construct_url(table_config, query_params):

    url_prefix = 'https://www.baseball-reference.com'

    if 'first_name' not in query_params or 'last_name' not in query_params: 
        raise ValueError(generate_name_required_error_message(query_params))

    first_name = query_params['first_name']
    last_name = query_params['last_name']

    if len(first_name) == 0 or len(first_name) == 0 : 
        raise ValueError(generate_name_required_error_message(query_params))

    player_href = individual_player_identifier_service.get_player_href(first_name, last_name)

    url = f'{url_prefix}{player_href}'

    print(f'url: {url}')
    return url

def generate_name_required_error_message(query_params): 

    requested_table = query_params['table']

    error_message_text = 'Value error in the incoming request payload:'
    name_error_text = f'The table requested ("{requested_table}") requires the players first name and last name to be specified in the query parameters.'
    report_text = f'If you believe this to be an error, please report this as an issue @ https://github.com/joesmi9/baseballdc.'

    return f'{error_message_text}\n\n{name_error_text}\n{report_text}\n';




from baseball_reference.team import baseball_reference_team_request_handler
from baseball_reference.season import baseball_referenece_season_request_handler
from baseball_reference.individual_player import baseball_reference_individual_player_request_handler

from baseball_reference.const import baseball_reference_scope_const

def get_baseball_reference_data(baseballdc_request):

    query_params = baseballdc_request['query_params'];

    validate_baseball_reference_params(query_params);

    if(query_params['scope'].upper() == baseball_reference_scope_const.INDIVIDUAL_PLAYER):
        df = baseball_reference_individual_player_request_handler.get_baseball_reference_individual_player_data(query_params)

    elif(query_params['scope'].upper() == baseball_reference_scope_const.TEAM):
        df = baseball_reference_team_request_handler.get_baseball_reference_team_data(query_params)
        
    elif(query_params['scope'].upper() == baseball_reference_scope_const.SEASON):
        df = baseball_referenece_season_request_handler.get_baseball_reference_season_data(query_params)

    return df

def validate_baseball_reference_params(baseball_reference_params):
    if(baseball_reference_params.get('scope').upper() not in baseball_reference_scope_const.BASEBALL_REFERENCE_SCOPE_LIST):
        raise ValueError(generate_scope_error_message(baseball_reference_params))


def generate_scope_error_message(baseball_reference_params): 

    requested_scope = baseball_reference_params['scope']
    valid_scopes= baseball_reference_scope_const.BASEBALL_REFERENCE_SCOPE_LIST

    error_message_text = 'Value error in the incoming request payload:'
    scope_error_text = f'The scope requested ("{requested_scope}") was not valid.'
    valid_scope_text = f'The current list of valid Baseball Reference scopes to choose from for basballdc is: {valid_scopes}'

    return f'{error_message_text}\n\n{scope_error_text}\n{valid_scope_text}\n';


def construct_url(table_config, query_params):

    url_prefix = table_config['url_prefix']

    url_team = ''
    if(table_config['team_required'] == True):

        if 'team' not in query_params:
            raise ValueError(generate_team_required_error_message(query_params))

        url_team = '/' + query_params['team']

    url_year = ''
    if(table_config['year_required'] == True):

        if 'year' not in query_params:
            raise ValueError(generate_year_required_error_message(query_params))

        url_year = '/' + str(query_params['year'])

    url_postfix = table_config['url_postfix']

    shtml_postfix = ''
    if(table_config['shtml_postfix_required'] == True):
        shtml_postfix = '.shtml'

    url = f'{url_prefix}{url_team}{url_year}{url_postfix}{shtml_postfix}'

    print(f'url: {url}')
    return url

def generate_team_required_error_message(query_params): 

    requested_table = query_params['table']

    error_message_text = 'Value error in the incoming request payload:'
    team_error_text = f'The table requested ("{requested_table}") requires a team to be specified in the query parameters.'
    report_text = f'If you believe this to be an error, please report this as an issue @ https://github.com/joesmi9/baseballdc.'

    return f'{error_message_text}\n\n{team_error_text}\n{report_text}\n';

def generate_year_required_error_message(query_params): 

    requested_table = query_params['table']

    error_message_text = 'Value error in the incoming request payload:'
    year_error_text = f'The table requested ("{requested_table}") requires a year to be specified in the query parameters.'
    report_text = f'If you believe this to be an error, please report this as an issue @ https://github.com/joesmi9/baseballdc.'

    return f'{error_message_text}\n\n{year_error_text}\n{report_text}\n';


from setuptools import setup, find_packages
 
setup(
  name='baseballdc',
  version='0.0.8',
  description='Baseball Data Center',
  author='Joe Smith',
  author_email='joesmi9.sde@gmail.com',
  url='https://github.com/joesmi9/baseballdc',
  license='MIT', 
  keywords=['baseball', 'baseball analytics', 'baseball reference'],
  packages=['baseballdc'],
  install_requires=[''],
  classifiers=[
    'Development Status :: 2 - Pre-Alpha',
    'Intended Audience :: Developers',
    'Programming Language :: Python :: 3',
    'Operating System :: Unix',
    'Operating System :: MacOS :: MacOS X',
    'Operating System :: Microsoft :: Windows',
  ]
)


import requests
from bs4 import BeautifulSoup

def get_player_href(first_name, last_name):

    last_name_first_letter = last_name[0].lower()

    url = f'https://www.baseball-reference.com/players/{last_name_first_letter}/'

    response = requests.get(url, headers = {
        'accept-language':'en-US,en;q=0.8',
    })

    soup = BeautifulSoup(response.content, 'html5lib')

    table = soup.find('div', {'id': 'div_players_'})

    name = f'{first_name} {last_name}' 

    href = None
    for para in table.find_all("p"):
        if name.upper() in para.get_text().upper(): 
            el = para.find(href=True)
            href = el['href']

    if(href == None):
        raise ValueError(generate_player_not_found_error_message(first_name, last_name))

    return href

def generate_player_not_found_error_message(first_name, last_name): 

    error_message_text = 'Value error in the incoming request payload:'
    not_found_error_text = f'Could not find the requested player on Baseball Reference ("{first_name} {last_name}")'
    report_text = f'If you believe this to be an error, please report this as an issue @ https://github.com/joesmi9/baseballdc.'

    return f'{error_message_text}\n\n{not_found_error_text}\n{report_text}\n';